# Generated by Django 3.1.4 on 2020-12-28 15:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('jet', '0002_delete_userdashboardmodule'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pinnedapplication',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
    ]



from django.contrib import admin
from django.test import TestCase, RequestFactory
from django.utils.encoding import smart_str
from jet.filters import RelatedFieldAjaxListFilter
from jet.tests.models import RelatedToTestModel, TestModel


from django.contrib.admin.utils import get_fields_from_path


class FiltersTestCase(TestCase):
    def setUp(self):
        self.models = []
        self.factory = RequestFactory()
        self.models.append(TestModel.objects.create(field1='first', field2=1))
        self.models.append(TestModel.objects.create(field1='second', field2=2))

    def get_related_field_ajax_list_filter_params(self):
        model = RelatedToTestModel
        field_path = 'field'
        field = get_fields_from_path(model, field_path)[-1]
        lookup_params = {}
        model_admin = admin.site._registry.get(model)

        return field, lookup_params, model, model_admin, field_path

    def get_related_field_ajax_list_filter_params_with_initial(self):
        model = RelatedToTestModel
        field_path = 'field'
        field = get_fields_from_path(model, field_path)[-1]
        lookup_params = {'field__id__exact':2}
        model_admin = admin.site._registry.get(model)

        return field, lookup_params, model, model_admin, field_path

    def test_related_field_ajax_list_filter(self):
        request = self.factory.get('url')
        field, lookup_params, model, model_admin, field_path = self.get_related_field_ajax_list_filter_params()
        list_filter = RelatedFieldAjaxListFilter(field, request, lookup_params, model, model_admin, field_path)

        self.assertTrue(list_filter.has_output())

        choices = list_filter.field_choices(field, request, model_admin)

        self.assertIsInstance(choices, list)
        self.assertEqual(len(choices), 0)

    def test_related_field_ajax_list_filter_with_initial(self):
        initial = self.models[1]
        request = self.factory.get('url', {'field__id__exact': initial.pk})
        field, lookup_params, model, model_admin, field_path = self.get_related_field_ajax_list_filter_params_with_initial()
        list_filter = RelatedFieldAjaxListFilter(field, request, lookup_params, model, model_admin, field_path)

        self.assertTrue(list_filter.has_output())

        choices = list_filter.field_choices(field, request, model_admin)

        self.assertIsInstance(choices, list)
        self.assertEqual(len(choices), 1)
        self.assertEqual(choices[0], (initial.pk, smart_str(initial)))




from django.contrib.auth.models import User
try:
    from django.core.urlresolvers import reverse
except ImportError: # Django 1.11
    from django.urls import reverse

from django.test import TestCase, Client
from jet.dashboard.dashboard import Dashboard
from jet.dashboard.modules import LinkList, RecentActions
from jet.dashboard.models import UserDashboardModule
from jet.tests.dashboard import TestIndexDashboard


class DashboardTestCase(TestCase):
    class Request:
        def __init__(self, user):
            self.user = user

    def setUp(self):
        self._login()
        self._init_dashboard()

    def _login(self):
        username = 'admin'
        email = 'admin@example.com'
        password = 'admin'
        self.admin = Client()
        self.admin_user = User.objects.create_superuser(username, email, password)
        return self.admin.login(username=username, password=password)

    def _init_dashboard(self):
        UserDashboardModule.objects.create(
            title='',
            module='jet.dashboard.modules.LinkList',
            app_label=None,
            user=self.admin_user,
            column=0,
            order=0
        )
        UserDashboardModule.objects.create(
            title='',
            module='jet.dashboard.modules.RecentActions',
            app_label=None,
            user=self.admin_user,
            column=0,
            order=1
        )
        self.dashboard = TestIndexDashboard({'request': self.Request(self.admin_user)})

    def test_custom_columns(self):
        self.assertEqual(self.dashboard.columns, 3)

    def test_init_with_context_called(self):
        self.assertTrue(self.dashboard.init_with_context_called)

    def test_load_modules(self):
        self.assertEqual(len(self.dashboard.modules), 2)
        self.assertTrue(isinstance(self.dashboard.modules[0], LinkList))
        self.assertTrue(isinstance(self.dashboard.modules[1], RecentActions))

    def test_media(self):
        media = self.dashboard.media()
        self.assertEqual(len(media.js), 2)
        self.assertEqual(media.js[0], 'file.js')
        self.assertEqual(media.js[1], 'file2.js')
        self.assertEqual(len(media.css), 2)
        self.assertEqual(media.css[0], 'file.css')
        self.assertEqual(media.css[1], 'file2.css')

    def test_index_dashboard_view(self):
        response = self.admin.get(reverse('admin:index'))
        self.assertEqual(response.status_code, 200)
        self.assertTrue('dashboard' in response.context)

        dashboard = response.context['dashboard']

        self.assertIsInstance(dashboard, Dashboard)
        self.assertIsNone(dashboard.app_label)

    def test_app_index_dashboard_view(self):
        app_label = 'tests'
        response = self.admin.get(reverse('admin:app_list', args=(app_label,)))
        self.assertEqual(response.status_code, 200)
        self.assertTrue('dashboard' in response.context)

        dashboard = response.context['dashboard']

        self.assertIsInstance(dashboard, Dashboard)
        self.assertEqual(dashboard.app_label, app_label)



# Generated by Django 3.2.12 on 2022-04-09 05:43

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    replaces = [('jet', '0001_initial'), ('jet', '0002_delete_userdashboardmodule'), ('jet', '0003_auto_20201228_1540'), ('jet', '0004_auto_20201228_1802')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Bookmark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(verbose_name='URL')),
                ('title', models.CharField(max_length=255, verbose_name='title')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='user')),
                ('date_add', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date created')),
            ],
            options={
                'ordering': ('date_add',),
                'verbose_name': 'bookmark',
                'verbose_name_plural': 'bookmarks',
            },
        ),
        migrations.CreateModel(
            name='PinnedApplication',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app_label', models.CharField(max_length=255, verbose_name='application name')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='user')),
                ('date_add', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date created')),
            ],
            options={
                'ordering': ('date_add',),
                'verbose_name': 'pinned application',
                'verbose_name_plural': 'pinned applications',
            },
        ),
    ]



from django.conf.urls import include
from django.contrib import admin

admin.autodiscover()

from django.urls import path

urlpatterns = [
        path(r'jet/', include('jet.urls', 'jet')),
        path(r'jet/dashboard/', include('jet.dashboard.urls', 'jet-dashboard')),
        path(r'admin/doc/', include('django.contrib.admindocs.urls')),
        path('admin/', admin.site.urls),
]




from importlib import import_module
from jet.dashboard import settings


def get_current_dashboard(location):
    if location == 'index':
        path = settings.JET_INDEX_DASHBOARD
    elif location == 'app_index':
        path = settings.JET_APP_INDEX_DASHBOARD
    else:
        raise ValueError('Unknown dashboard location: %s' % location)

    module, cls = path.rsplit('.', 1)

    try:
        module = import_module(module)
        index_dashboard_cls = getattr(module, cls)
        return index_dashboard_cls
    except ImportError:
        return None



import json
from django.contrib.auth.models import User
try:
    from django.core.urlresolvers import reverse
except ImportError: # Django 1.11
    from django.urls import reverse

from django.test import TestCase, Client
from jet.dashboard.modules import LinkList
from jet.models import Bookmark
from jet.dashboard.models import UserDashboardModule


class ViewsTestCase(TestCase):
    def setUp(self):
        self.assertTrue(self._login())

    def _login(self):
        username = 'admin'
        email = 'admin@example.com'
        password = 'admin'
        self.admin = Client()
        self.admin_user = User.objects.create_superuser(username, email, password)
        return self.admin.login(username=username, password=password)

    def test_module_update_view(self):
        title = 'Quick links Test'
        new_title = title + '2'
        new_layout = 'stacked'
        module = UserDashboardModule.objects.create(
            title=title,
            module='jet.dashboard.modules.LinkList',
            app_label=None,
            user=self.admin_user,
            column=0,
            order=0,
            settings='{"layout": "inline"}',
            children='[]'
        )

        response = self.admin.get(reverse('jet-dashboard:update_module', kwargs={'pk': module.pk}))
        self.assertEqual(response.status_code, 200)
        self.assertTrue(response.context['module'] is not None)
        self.assertTrue(isinstance(response.context['module'], LinkList))
        self.assertEqual(response.context['module'].title, title)

        post = {
            'title': new_title,
            'layout': new_layout,
            'children-TOTAL_FORMS': '2',
            'children-INITIAL_FORMS': '1',
            'children-MIN_NUM_FORMS': '0',
            'children-MAX_NUM_FORMS': '1000',
            'children-0-url': 'http://docs.djangoproject.com/',
            'children-0-title': 'Django documentation',
            'children-0-external': 'on',
            'children-1-url': '',
            'children-1-title': '',
            'children-__prefix__-url': '',
            'children-__prefix__-title': '',
            '_save': 'Save'
        }

        self.admin.post(reverse('jet-dashboard:update_module', kwargs={'pk': module.pk}), post)
        self.assertEqual(response.status_code, 200)
        module = UserDashboardModule.objects.get(pk=module.pk)
        settings = json.loads(module.settings)
        self.assertEqual(module.title, new_title)
        self.assertEqual(settings['layout'], new_layout)

        module.delete()

    def test_add_bookmark_view(self):
        url = 'http://test.com/'
        title = 'Title'
        response = self.admin.post(reverse('jet:add_bookmark'), {'url': url, 'title': title})
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertNotEqual(response['id'], None)
        bookmark = Bookmark.objects.get(pk=response['id'])
        self.assertNotEqual(bookmark, None)
        self.assertEqual(bookmark.title, title)
        self.assertEqual(bookmark.url, url)

    def test_add_bookmark_view_unauthorized(self):
        url = 'http://test.com/'
        title = 'Title'
        response = self.client.post(reverse('jet:add_bookmark'), {'url': url, 'title': title})
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertTrue(response['error'])

    def test_remove_bookmark_view(self):
        url = 'http://test.com/'
        title = 'Title'
        bookmark = Bookmark.objects.create(url=url, title=title, user=self.admin_user)
        response = self.admin.post(reverse('jet:remove_bookmark'), {'id': bookmark.id})
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertFalse(Bookmark.objects.filter(pk=bookmark.pk).exists())

    def test_toggle_application_pin_view(self):
        app_label = 'test_app'

        response = self.admin.post(reverse('jet:toggle_application_pin'), {'app_label': app_label})
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertTrue(response['pinned'])

        response = self.admin.post(reverse('jet:toggle_application_pin'), {'app_label': app_label})
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertFalse(response['pinned'])

    def test_update_dashboard_modules_view(self):
        app_label = None
        module_0 = UserDashboardModule.objects.create(
            title='',
            module='jet.dashboard.modules.LinkList',
            app_label=app_label,
            user=self.admin_user,
            column=0,
            order=0
        )
        module_1 = UserDashboardModule.objects.create(
            title='',
            module='jet.dashboard.modules.LinkList',
            app_label=app_label,
            user=self.admin_user,
            column=0,
            order=1
        )
        response = self.admin.post(reverse('jet-dashboard:update_dashboard_modules'), {
            'app_label': '',
            'modules': json.dumps([
                {'id': module_0.pk, 'column': 0, 'order': 1},
                {'id': module_1.pk, 'column': 0, 'order': 0}
            ])
        })

        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])

        module_0 = UserDashboardModule.objects.get(pk=module_0.pk)
        module_1 = UserDashboardModule.objects.get(pk=module_1.pk)

        self.assertEqual(module_0.order, 1)
        self.assertEqual(module_1.order, 0)

        module_0.delete()
        module_1.delete()

    def test_add_user_dashboard_module_view(self):
        response = self.admin.post(reverse('jet-dashboard:add_user_dashboard_module'), {
            'app_label': '',
            'type': 'available_children',
            'module': 0
        })

        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertNotEqual(response['id'], None)
        module = UserDashboardModule.objects.get(pk=response['id'])
        self.assertNotEqual(module, None)

    def test_add_user_app_dashboard_module_view(self):
        app_label = 'auth'
        response = self.admin.post(reverse('jet-dashboard:add_user_dashboard_module'), {
            'app_label': app_label,
            'type': 'available_children',
            'module': 0
        })

        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertNotEqual(response['id'], None)
        module = UserDashboardModule.objects.get(pk=response['id'])
        self.assertNotEqual(module, None)
        self.assertEqual(module.app_label, app_label)

    def test_update_dashboard_module_collapse_view(self):
        module = UserDashboardModule.objects.create(
            title='',
            module='jet.dashboard.modules.LinkList',
            app_label=None,
            user=self.admin_user,
            column=0,
            order=0
        )
        response = self.admin.post(reverse('jet-dashboard:update_dashboard_module_collapse'), {
            'id': module.pk, 'collapsed': True
        })
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertTrue(response['collapsed'])

        module = UserDashboardModule.objects.get(pk=module.pk)
        response = self.admin.post(reverse('jet-dashboard:update_dashboard_module_collapse'), {
            'id': module.pk, 'collapsed': False
        })
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertFalse(response['collapsed'])

        module.delete()

    def test_remove_dashboard_module_view(self):
        module = UserDashboardModule.objects.create(
            title='',
            module='jet.dashboard.modules.LinkList',
            app_label=None,
            user=self.admin_user,
            column=0,
            order=0
        )
        response = self.admin.post(reverse('jet-dashboard:remove_dashboard_module'), {'id': module.pk})
        self.assertEqual(response.status_code, 200)
        response = json.loads(response.content.decode())
        self.assertFalse(response['error'])
        self.assertFalse(UserDashboardModule.objects.filter(pk=module.pk).exists())



import os
import django
from django.conf import global_settings

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = '!DJANGO_JET_TESTS!'

DEBUG = True
TEMPLATE_DEBUG = DEBUG
DEBUG_PROPAGATE_EXCEPTIONS = True

ROOT_URLCONF = 'jet.tests.urls'

INSTALLED_APPS = (
    'jet.dashboard',
    'jet',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.admin',
    'jet.tests',
)

MIDDLEWARE = MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

if django.VERSION[:2] < (1, 9):
    TEMPLATE_CONTEXT_PROCESSORS = tuple(global_settings.TEMPLATE_CONTEXT_PROCESSORS) + (
        'django.core.context_processors.request',
    )
else:
    TEMPLATES = [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [os.path.join(BASE_DIR, 'templates')],
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': (
                    'django.template.context_processors.debug',
                    'django.template.context_processors.request',
                    'django.contrib.auth.context_processors.auth',
                    'django.contrib.messages.context_processors.messages',
                )
            },
        },
    ]

DATABASES = {
    'default': {
        'NAME': "db",
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

TIME_ZONE = 'UTC'
LANGUAGE_CODE = 'en-us'
USE_I18N = True
USE_L10N = True

MEDIA_ROOT = ''
MEDIA_URL = ''

STATIC_URL = '/static/'

JET_INDEX_DASHBOARD = 'jet.tests.dashboard.TestIndexDashboard'
JET_APP_INDEX_DASHBOARD = 'jet.tests.dashboard.TestAppIndexDashboard'



import datetime
import json
from collections import OrderedDict
from django.template import Context
from django.utils import translation
from jet import settings
from jet.models import PinnedApplication

try:
    from django.apps.registry import apps
except ImportError:
    try:
        from django.apps import apps # Fix Django 1.7 import issue
    except ImportError:
        pass
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
try:
    from django.core.urlresolvers import reverse, resolve, NoReverseMatch
except ImportError: # Django 1.11
    from django.urls import reverse, resolve, NoReverseMatch

from django.contrib.admin import AdminSite
from django.utils.encoding import smart_str
from django.utils.text import capfirst
from django.contrib import messages
from django.utils.encoding import force_str
from django.utils.functional import Promise
from django.contrib.admin.options import IncorrectLookupParameters
from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.utils.text import slugify


class JsonResponse(HttpResponse):
    """
    An HTTP response class that consumes data to be serialized to JSON.
    :param data: Data to be dumped into json. By default only ``dict`` objects
      are allowed to be passed due to a security flaw before EcmaScript 5. See
      the ``safe`` parameter for more information.
    :param encoder: Should be an json encoder class. Defaults to
      ``django.core.serializers.json.DjangoJSONEncoder``.
    :param safe: Controls if only ``dict`` objects may be serialized. Defaults
      to ``True``.
    """

    def __init__(self, data, encoder=DjangoJSONEncoder, safe=True, **kwargs):
        if safe and not isinstance(data, dict):
            raise TypeError('In order to allow non-dict objects to be '
                'serialized set the safe parameter to False')
        kwargs.setdefault('content_type', 'application/json')
        data = json.dumps(data, cls=encoder)
        super(JsonResponse, self).__init__(content=data, **kwargs)


def get_app_list(context, order=True):
    admin_site = get_admin_site(context)
    request = context['request']

    app_dict = {}
    for model, model_admin in admin_site._registry.items():
        app_label = model._meta.app_label
        try:
            has_module_perms = model_admin.has_module_permission(request)
        except AttributeError:
            has_module_perms = request.user.has_module_perms(app_label) # Fix Django < 1.8 issue

        if has_module_perms:
            perms = model_admin.get_model_perms(request)

            # Check whether user has any perm for this module.
            # If so, add the module to the model_list.
            if True in perms.values():
                info = (app_label, model._meta.model_name)
                model_dict = {
                    'name': capfirst(model._meta.verbose_name_plural),
                    'object_name': model._meta.object_name,
                    'perms': perms,
                    'model_name': model._meta.model_name
                }
                if perms.get('view', False):
                    try:
                        model_dict['admin_url'] = reverse('admin:%s_%s_changelist' % info, current_app=admin_site.name)
                    except NoReverseMatch:
                        pass
                if perms.get('add', False):
                    try:
                        model_dict['add_url'] = reverse('admin:%s_%s_add' % info, current_app=admin_site.name)
                    except NoReverseMatch:
                        pass
                if app_label in app_dict:
                    app_dict[app_label]['models'].append(model_dict)
                else:
                    try:
                        name = apps.get_app_config(app_label).verbose_name
                    except NameError:
                        name = app_label.title()
                    app_dict[app_label] = {
                        'name': name,
                        'app_label': app_label,
                        'app_url': reverse(
                            'admin:app_list',
                            kwargs={'app_label': app_label},
                            current_app=admin_site.name,
                        ),
                        'has_module_perms': has_module_perms,
                        'models': [model_dict],
                    }

    # Sort the apps alphabetically.
    app_list = list(app_dict.values())

    if order:
        app_list.sort(key=lambda x: x['name'].lower())

        # Sort the models alphabetically within each app.
        for app in app_list:
            app['models'].sort(key=lambda x: x['name'])

    return app_list


def get_admin_site(context):
    try:
        current_resolver = resolve(context.get('request').path)
        index_resolver = resolve(reverse('%s:index' % current_resolver.namespaces[0]))

        if hasattr(index_resolver.func, 'admin_site'):
            return index_resolver.func.admin_site

        for func_closure in index_resolver.func.__closure__:
            if isinstance(func_closure.cell_contents, AdminSite):
                return func_closure.cell_contents
    except:
        pass

    return admin.site


def get_admin_site_name(context):
    return get_admin_site(context).name


class LazyDateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime) or isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, Promise):
            return force_str(obj)
        return self.encode(obj)


def get_model_instance_label(instance):
    if getattr(instance, "related_label", None):
        return instance.related_label()
    return smart_str(instance)


class SuccessMessageMixin(object):
    """
    Adds a success message on successful form submission.
    """
    success_message = ''

    def form_valid(self, form):
        response = super(SuccessMessageMixin, self).form_valid(form)
        success_message = self.get_success_message(form.cleaned_data)
        if success_message:
            messages.success(self.request, success_message)
        return response

    def get_success_message(self, cleaned_data):
        return self.success_message % cleaned_data


def get_model_queryset(admin_site, model, request, preserved_filters=None):
    model_admin = admin_site._registry.get(model)

    if model_admin is None:
        return

    try:
        changelist_url = reverse('%s:%s_%s_changelist' % (
            admin_site.name,
            model._meta.app_label,
            model._meta.model_name
        ))
    except NoReverseMatch:
        return

    changelist_filters = None

    if preserved_filters:
        changelist_filters = preserved_filters.get('_changelist_filters')

    if changelist_filters:
        changelist_url += '?' + changelist_filters

    if model_admin:
        queryset = model_admin.get_queryset(request)
    else:
        queryset = model.objects

    list_display = model_admin.get_list_display(request)
    list_display_links = model_admin.get_list_display_links(request, list_display)
    list_filter = model_admin.get_list_filter(request)
    search_fields = model_admin.get_search_fields(request) \
        if hasattr(model_admin, 'get_search_fields') else model_admin.search_fields
    list_select_related = model_admin.get_list_select_related(request) \
        if hasattr(model_admin, 'get_list_select_related') else model_admin.list_select_related



    actions = model_admin.get_actions(request)
    if actions:
        list_display = ['action_checkbox'] + list(list_display)

    ChangeList = model_admin.get_changelist(request)

    change_list_args = [
        request, model, list_display, list_display_links, list_filter,
        model_admin.date_hierarchy, search_fields,  list_select_related,
        model_admin.list_per_page, model_admin.list_max_show_all,
        model_admin.list_editable, model_admin]


    try:
        sortable_by = model_admin.get_sortable_by(request)
        change_list_args.append(sortable_by)
    except AttributeError:
        # django version < 2.1
        pass

    try:
        search_help_text = model_admin.get_sortable_by(request) \
            if hasattr(model_admin, 'get_search_help_text') else model_admin.search_help_text
        change_list_args.append(search_help_text)
    except AttributeError:
        # django version < 4.0
        pass


    try:
        cl = ChangeList(*change_list_args)
        queryset = cl.get_queryset(request)
    except IncorrectLookupParameters:
        pass

    return queryset


def get_possible_language_codes():
    language_code = translation.get_language()

    language_code = language_code.replace('_', '-').lower()
    language_codes = []

    # making dialect part uppercase
    split = language_code.split('-', 2)
    if len(split) == 2:
        language_code = '%s-%s' % (split[0].lower(), split[1].upper()) if split[0] != split[1] else split[0]

    language_codes.append(language_code)

    # adding language code without dialect part
    if len(split) == 2:
        language_codes.append(split[0].lower())

    return language_codes


def get_original_menu_items(context):
    if context.get('user') and user_is_authenticated(context['user']):
        pinned_apps = PinnedApplication.objects.filter(user=context['user'].pk).values_list('app_label', flat=True)
    else:
        pinned_apps = []

    original_app_list = get_app_list(context)

    return map(lambda app: {
        'app_label': app['app_label'],
        'url': app['app_url'],
        'url_blank': False,
        'label': app.get('name', capfirst(_(app['app_label']))),
        'has_perms': app.get('has_module_perms', False),
        'models': list(map(lambda model: {
            'url': model.get('admin_url'),
            'url_blank': False,
            'name': model['model_name'],
            'object_name': model['object_name'],
            'label': model.get('name', model['object_name']),
            'has_perms': any(model.get('perms', {}).values()),
        }, app['models'])),
        'pinned': app['app_label'] in pinned_apps,
        'custom': False
    }, original_app_list)


def get_menu_item_url(url, original_app_list):
    if isinstance(url, dict):
        url_type = url.get('type')

        if url_type == 'app':
            return original_app_list[url['app_label']]['url']
        elif url_type == 'model':
            models = dict(map(
                lambda x: (x['name'], x['url']),
                original_app_list[url['app_label']]['models']
            ))
            return models[url['model']]
        elif url_type == 'reverse':
            return reverse(url['name'], args=url.get('args'), kwargs=url.get('kwargs'))
    elif isinstance(url, str):
        return url


def get_menu_items(context):
    pinned_apps = PinnedApplication.objects.filter(user=context['user'].pk).values_list('app_label', flat=True)
    original_app_list = OrderedDict(map(lambda app: (app['app_label'], app), get_original_menu_items(context)))
    custom_app_list = settings.JET_SIDE_MENU_ITEMS
    custom_app_list_deprecated = settings.JET_SIDE_MENU_CUSTOM_APPS

    if custom_app_list not in (None, False):
        if isinstance(custom_app_list, dict):
            admin_site = get_admin_site(context)
            custom_app_list = custom_app_list.get(admin_site.name, [])

        app_list = []

        def get_menu_item_app_model(app_label, data):
            item = {'has_perms': True}

            if 'name' in data:
                parts = data['name'].split('.', 2)

                if len(parts) > 1:
                    app_label, name = parts
                else:
                    name = data['name']

                if app_label in original_app_list:
                    models = dict(map(
                        lambda x: (x['name'], x),
                        original_app_list[app_label]['models']
                    ))

                    if name in models:
                        item = models[name].copy()

            if 'label' in data:
                item['label'] = data['label']

            if 'url' in data:
                item['url'] = get_menu_item_url(data['url'], original_app_list)

            if 'url_blank' in data:
                item['url_blank'] = data['url_blank']

            if 'permissions' in data:
                item['has_perms'] = item.get('has_perms', True) and context['user'].has_perms(data['permissions'])

            return item

        def get_menu_item_app(data):
            app_label = data.get('app_label')

            if not app_label:
                if 'label' not in data:
                    raise Exception('Custom menu items should at least have \'label\' or \'app_label\' key')
                app_label = 'custom_%s' % slugify(data['label'], allow_unicode=True)

            if app_label in original_app_list:
                item = original_app_list[app_label].copy()
            else:
                item = {'app_label': app_label, 'has_perms': True}

            if 'label' in data:
                item['label'] = data['label']

            if 'items' in data:
                item['items'] = list(map(lambda x: get_menu_item_app_model(app_label, x), data['items']))

            if 'url' in data:
                item['url'] = get_menu_item_url(data['url'], original_app_list)

            if 'url_blank' in data:
                item['url_blank'] = data['url_blank']

            if 'permissions' in data:
                item['has_perms'] = item.get('has_perms', True) and context['user'].has_perms(data['permissions'])

            item['pinned'] = item['app_label'] in pinned_apps

            return item

        for data in custom_app_list:
            item = get_menu_item_app(data)
            app_list.append(item)
    elif custom_app_list_deprecated not in (None, False):
        app_dict = {}
        models_dict = {}

        for app in original_app_list.values():
            app_label = app['app_label']
            app_dict[app_label] = app

            for model in app['models']:
                if app_label not in models_dict:
                    models_dict[app_label] = {}

                models_dict[app_label][model['object_name']] = model

            app['items'] = []

        app_list = []

        if isinstance(custom_app_list_deprecated, dict):
            admin_site = get_admin_site(context)
            custom_app_list_deprecated = custom_app_list_deprecated.get(admin_site.name, [])

        for item in custom_app_list_deprecated:
            app_label, models = item

            if app_label in app_dict:
                app = app_dict[app_label]

                for model_label in models:
                    if model_label == '__all__':
                        app['items'] = models_dict[app_label].values()
                        break
                    elif model_label in models_dict[app_label]:
                        model = models_dict[app_label][model_label]
                        app['items'].append(model)

                app_list.append(app)
    else:
        def map_item(item):
            item['items'] = item['models']
            return item
        app_list = list(map(map_item, original_app_list.values()))

    current_found = False

    for app in app_list:
        if not current_found:
            for model in app['items']:
                if not current_found and model.get('url') and context['request'].path.startswith(model['url']):
                    model['current'] = True
                    current_found = True
                else:
                    model['current'] = False

            if not current_found and app.get('url') and context['request'].path.startswith(app['url']):
                app['current'] = True
                current_found = True
            else:
                app['current'] = False

    return app_list


def context_to_dict(context):
    if isinstance(context, Context):
        flat = {}
        for d in context.dicts:
            flat.update(d)
        context = flat

    return context


def user_is_authenticated(user):
    if not hasattr(user.is_authenticated, '__call__'):
        return user.is_authenticated
    else:
        return user.is_authenticated()



from django import template
from jet.dashboard.utils import get_current_dashboard

register = template.Library()
assignment_tag = register.assignment_tag if hasattr(register, 'assignment_tag') else register.simple_tag


@assignment_tag(takes_context=True)
def get_dashboard(context, location):
    dashboard_cls = get_current_dashboard(location)

    app_label = context['request'].resolver_match.kwargs.get('app_label')

    return dashboard_cls(context, app_label=app_label)


@register.filter
def format_change_message(log_entry):
    # Django 1.10+
    if hasattr(log_entry, 'get_change_message'):
        return log_entry.get_change_message()
    else:
        return log_entry.change_message



from django.contrib import messages
from django.core.exceptions import ValidationError
try:
    from django.core.urlresolvers import reverse
except ImportError: # Django 1.11
    from django.urls import reverse

from django.forms.formsets import formset_factory
from django.http import HttpResponseRedirect
from django.views.decorators.http import require_POST, require_GET
from jet.dashboard.forms import UpdateDashboardModulesForm, AddUserDashboardModuleForm, \
    UpdateDashboardModuleCollapseForm, RemoveDashboardModuleForm, ResetDashboardForm
from jet.dashboard.models import UserDashboardModule
from jet.utils import JsonResponse, get_app_list, SuccessMessageMixin, user_is_authenticated
from django.views.generic import UpdateView
from django.utils.translation import gettext_lazy as _


class UpdateDashboardModuleView(SuccessMessageMixin, UpdateView):
    model = UserDashboardModule
    fields = ('title',)
    template_name = 'jet.dashboard/update_module.html'
    success_message = _('Widget was successfully updated')
    object = None
    module = None

    def has_permission(self, request):
        return request.user.is_active and request.user.is_staff

    def get_success_url(self):
        if self.object.app_label:
            return reverse('admin:app_list', kwargs={'app_label': self.object.app_label})
        else:
            return reverse('admin:index')

    def get_module(self):
        object = self.object if getattr(self, 'object', None) is not None else self.get_object()
        return object.load_module()

    def get_settings_form_kwargs(self):
        kwargs = {
            'initial': self.module.settings
        }

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
        return kwargs

    def get_settings_form(self):
        if self.module.settings_form:
            form = self.module.settings_form(**self.get_settings_form_kwargs())
            if hasattr(form, 'set_module'):
                form.set_module(self.module)
            return form

    def get_children_formset_kwargs(self):
        kwargs = {
            'initial': self.module.children,
            'prefix': 'children',
        }

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
        return kwargs

    def get_children_formset(self):
        if self.module.child_form:
            return formset_factory(self.module.child_form, can_delete=True, extra=1)(**self.get_children_formset_kwargs())

    def clean_children_data(self, children):
        children = list(filter(
            lambda item: isinstance(item, dict) and item and item.get('DELETE') is not True,
            children
        ))
        for item in children:
            item.pop('DELETE')
        return children

    def get_current_app(self):
        app_list = get_app_list({'request': self.request})

        for app in app_list:
            if app.get('app_label', app.get('name')) == self.object.app_label:
                return app

    def get_context_data(self, **kwargs):
        data = super(UpdateDashboardModuleView, self).get_context_data(**kwargs)
        data['title'] = _('Change')
        data['module'] = self.module
        data['settings_form'] = self.get_settings_form()
        data['children_formset'] = self.get_children_formset()
        data['child_name'] = self.module.child_name if self.module.child_name else _('Items')
        data['child_name_plural'] = self.module.child_name_plural if self.module.child_name_plural else _('Items')
        data['app'] = self.get_current_app()
        return data

    def dispatch(self, request, *args, **kwargs):
        if not self.has_permission(request):
            index_path = reverse('admin:index')
            return HttpResponseRedirect(index_path)

        self.object = self.get_object()
        self.module = self.get_module()(model=self.object)
        return super(UpdateDashboardModuleView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        settings_form = self.get_settings_form()
        children_formset = self.get_children_formset()

        data = request.POST.copy()

        if settings_form:
            if settings_form.is_valid():
                settings = settings_form.cleaned_data
                data['settings'] = self.module.dump_settings(settings)
            else:
                return self.form_invalid(self.get_form(self.get_form_class()))

        if children_formset:
            if children_formset.is_valid():
                self.module.children = self.clean_children_data(children_formset.cleaned_data)
                data['children'] = self.module.dump_children()
            else:
                return self.form_invalid(self.get_form(self.get_form_class()))

        request.POST = data

        return super(UpdateDashboardModuleView, self).post(request, *args, **kwargs)

    def form_valid(self, form):
        if 'settings' in form.data:
            form.instance.settings = form.data['settings']
        if 'children' in form.data:
            form.instance.children = form.data['children']
        return super(UpdateDashboardModuleView, self).form_valid(form)


@require_POST
def update_dashboard_modules_view(request):
    result = {'error': False}
    form = UpdateDashboardModulesForm(request, request.POST)

    if form.is_valid():
        form.save()
    else:
        result['error'] = True

    return JsonResponse(result)


@require_POST
def add_user_dashboard_module_view(request):
    result = {'error': False}
    form = AddUserDashboardModuleForm(request, request.POST)

    if form.is_valid():
        module = form.save()
        result['id'] = module.pk
        messages.success(request, _('Widget has been successfully added'))

        if module.app_label:
            result['success_url'] = reverse('admin:app_list', kwargs={'app_label': module.app_label})
        else:
            result['success_url'] = reverse('admin:index')
    else:
        result['error'] = True

    return JsonResponse(result)


@require_POST
def update_dashboard_module_collapse_view(request):
    result = {'error': False}

    try:
        instance = UserDashboardModule.objects.get(pk=request.POST.get('id'))
        form = UpdateDashboardModuleCollapseForm(request, request.POST, instance=instance)

        if form.is_valid():
            module = form.save()
            result['collapsed'] = module.collapsed
        else:
            result['error'] = True
    except UserDashboardModule.DoesNotExist:
        result['error'] = True

    return JsonResponse(result)


@require_POST
def remove_dashboard_module_view(request):
    result = {'error': False}

    try:
        instance = UserDashboardModule.objects.get(pk=request.POST.get('id'))
        form = RemoveDashboardModuleForm(request, request.POST, instance=instance)

        if form.is_valid():
            form.save()
        else:
            result['error'] = True
    except UserDashboardModule.DoesNotExist:
        result['error'] = True

    return JsonResponse(result)


@require_GET
def load_dashboard_module_view(request, pk):
    result = {'error': False}

    try:
        if not user_is_authenticated(request.user) or not request.user.is_staff:
            raise ValidationError('error')

        instance = UserDashboardModule.objects.get(pk=pk, user=request.user)
        module_cls = instance.load_module()
        module = module_cls(model=instance, context={'request': request})
        result['html'] = module.render()
    except (ValidationError, UserDashboardModule.DoesNotExist):
        result['error'] = True

    return JsonResponse(result)


@require_POST
def reset_dashboard_view(request):
    result = {'error': False}
    form = ResetDashboardForm(request, request.POST)

    if form.is_valid():
        form.save()
    else:
        result['error'] = True

    return JsonResponse(result)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Bookmark'
        db.create_table(u'jet_bookmark', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('user', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('date_add', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
        ))
        db.send_create_signal(u'jet', ['Bookmark'])

        # Adding model 'PinnedApplication'
        db.create_table(u'jet_pinnedapplication', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('app_label', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('user', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('date_add', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
        ))
        db.send_create_signal(u'jet', ['PinnedApplication'])

        # Adding model 'UserDashboardModule'
        db.create_table(u'jet_userdashboardmodule', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('module', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('app_label', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('user', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('column', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('order', self.gf('django.db.models.fields.IntegerField')()),
            ('settings', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('children', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('collapsed', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'jet', ['UserDashboardModule'])


    def backwards(self, orm):
        # Deleting model 'Bookmark'
        db.delete_table(u'jet_bookmark')

        # Deleting model 'PinnedApplication'
        db.delete_table(u'jet_pinnedapplication')

        # Deleting model 'UserDashboardModule'
        db.delete_table(u'jet_userdashboardmodule')


    models = {
        u'jet.bookmark': {
            'Meta': {'ordering': "('date_add',)", 'object_name': 'Bookmark'},
            'date_add': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        u'jet.pinnedapplication': {
            'Meta': {'ordering': "('date_add',)", 'object_name': 'PinnedApplication'},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'date_add': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        u'jet.userdashboardmodule': {
            'Meta': {'ordering': "('column', 'order')", 'object_name': 'UserDashboardModule'},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'children': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'collapsed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'column': ('django.db.models.fields.PositiveIntegerField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'module': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'order': ('django.db.models.fields.IntegerField', [], {}),
            'settings': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'user': ('django.db.models.fields.PositiveIntegerField', [], {})
        }
    }

    complete_apps = ['jet']


from django.db import models


class TestModel(models.Model):
    field1 = models.CharField(max_length=255)
    field2 = models.IntegerField()

    def __str__(self):
        return '%s%d' % (self.field1, self.field2)


class RelatedToTestModel(models.Model):
    field = models.ForeignKey(TestModel, on_delete=models.CASCADE)

    def __str__(self):
        return self.field


class SearchableTestModel(models.Model):
    field1 = models.CharField(max_length=255)
    field2 = models.IntegerField()

    def __str__(self):
        return '%s%d' % (self.field1, self.field2)

    @staticmethod
    def autocomplete_search_fields():
        return 'field1'



from django.test import TestCase
import pickle
from jet.ordered_set import OrderedSet


class OrderedSetTestCase(TestCase):
    def test_pickle(self):
        set1 = OrderedSet('abracadabra')
        roundtrip = pickle.loads(pickle.dumps(set1))
        self.assertEqual(roundtrip, set1)

    def test_empty_pickle(self):
        empty_oset = OrderedSet()
        empty_roundtrip = pickle.loads(pickle.dumps(empty_oset))
        self.assertEqual(empty_roundtrip, empty_oset)

    def test_order(self):
        set1 = OrderedSet('abracadabra')
        self.assertEqual(len(set1), 5)
        self.assertEqual(set1, OrderedSet(['a', 'b', 'r', 'c', 'd']))
        self.assertEqual(list(reversed(set1)), ['d', 'c', 'r', 'b', 'a'])

    def test_binary_operations(self):
        set1 = OrderedSet('abracadabra')
        set2 = OrderedSet('simsalabim')

        self.assertNotEqual(set1, set2)
        self.assertEqual(set1 & set2, OrderedSet(['a', 'b']))
        self.assertEqual(set1 | set2, OrderedSet(['a', 'b', 'r', 'c', 'd', 's', 'i', 'm', 'l']))
        self.assertEqual(set1 - set2, OrderedSet(['r', 'c', 'd']))

    def test_remove(self):
        set1 = OrderedSet('abracadabra')

        set1.remove('a')
        set1.remove('b')

        self.assertEqual(set1, OrderedSet('rcd'))
        self.assertNotIn('a', set1)
        self.assertNotIn('b', set1)
        self.assertIn('r', set1)

        set1.discard('a')
        set1.discard('a')

    def test_remove_error(self):
        set1 = OrderedSet('abracadabra')
        try:
            set1.remove('z')
        except KeyError:
            pass

    def test_clear(self):
        set1 = OrderedSet('abracadabra')
        set1.clear()

        self.assertEqual(len(set1), 0)
        self.assertEqual(set1, OrderedSet())

    def test_pop(self):
        set1 = OrderedSet('ab')
        elem = set1.pop()

        self.assertEqual(elem, 'b')
        elem = set1.pop()

        self.assertEqual(elem, 'a')
        self.assertRaises(KeyError, set1.pop)



from datetime import datetime, date
import json
from django.contrib.admin import AdminSite
from django.test import TestCase
from jet.tests.models import TestModel
from jet.utils import JsonResponse, get_model_instance_label, get_app_list, get_admin_site, LazyDateTimeEncoder


class UtilsTestCase(TestCase):
    def test_json_response(self):
        response = JsonResponse({'str': 'string', 'int': 1})
        response_dict = json.loads(response.content.decode())
        expected_dict = {"int": 1, "str": "string"}
        self.assertEqual(response_dict, expected_dict)
        self.assertEqual(response.get('Content-Type'), 'application/json')

    def test_get_model_instance_label(self):
        field1 = 'value'
        field2 = 2
        pinned_application = TestModel.objects.create(field1=field1, field2=field2)
        self.assertEqual(get_model_instance_label(pinned_application), '%s%d' % (field1, field2))

    def test_get_app_list(self):
        class User:
            is_active = True
            is_staff = True

            def has_module_perms(self, app):
                return True

            def has_perm(self, object):
                return True

        class Request:
            user = User()

        app_list = get_app_list({
            'request': Request(),
            'user': None
        })

        self.assertIsInstance(app_list, list)

        for app in app_list:
            self.assertIsInstance(app, dict)
            self.assertIsNotNone(app, app.get('models'))
            self.assertIsNotNone(app, app.get('app_url'))
            self.assertIsNotNone(app, app.get('app_label'))

            for model in app['models']:
                self.assertIsNotNone(app, model.get('object_name'))
                self.assertIsNotNone(app, model.get('name'))

    def test_get_admin_site(self):
        admin_site = get_admin_site({})
        self.assertIsInstance(admin_site, AdminSite)

    def test_lazy_date_time_encoder_dates(self):
        encoder = LazyDateTimeEncoder()

        ts = datetime.now()
        self.assertEqual(encoder.encode(ts), '"%s"' % ts.isoformat())

        ts = date(2015, 5, 3)
        self.assertEqual(encoder.encode(ts), '"%s"' % ts.isoformat())

    def test_lazy_date_time_encoder_dict(self):
        encoder = LazyDateTimeEncoder()
        self.assertEqual(encoder.encode({'key': 1}), '{"key": 1}')




import os
from setuptools import setup, find_packages


def read(fname):
    path = os.path.join(os.path.dirname(__file__), fname)
    try:
        file = open(path, encoding='utf-8')
    except TypeError:
        file = open(path)
    return file.read()


def get_install_requires():
    install_requires = ['Django']
    return install_requires


setup(
    name='django-jet-reboot',
    version=__import__('jet').VERSION,
    description='Modern template for Django admin interface with improved functionality',
    long_description=read('README.rst'),
    author='Denis Kildishev && assem-ch',
    url='https://github.com/assem-ch/django-jet-reboot',
    packages=find_packages(),
    license='AGPLv3',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'License :: Free for non-commercial use',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.9',
        'Environment :: Web Environment',
        'Topic :: Software Development',
        'Topic :: Software Development :: User Interfaces',
    ],
    zip_safe=False,
    include_package_data=True,
    install_requires=get_install_requires()
)
